# -*- coding: utf-8 -*-
import codecs

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.conf import settings

from django.http import Http404
from django.shortcuts import render, get_object_or_404
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from portailuser.models import PortailUser, Ecole
from ldap_inp import LdapClub, L
from club.models import Club, ArticleBlog


PAGES = {
    "arts": ["culture"],
    "sports": ["sport"],
    "technique": ["tech"],
    "bureaux": ["bureaux"],
    "listes": ["liste"],
    "autres": ["autre"],
}

TITRES = {
    "culture": "Arts et culture",
    "sport": "Sports",
    "tech": "Technique",
    "bureaux": "Bureaux de l'AE",
    "liste": "Listes",
    "autre": "Et aussi…",
}

TYPES = {
    None: "Club",
    "club": "Club",
    "asso": "Association",
}


def responsable(uids, titre, lien_annuaire, affiche_ecole):
    if uids is None or uids == []:
        return None

    if not isinstance(uids, list):
        uids = [uids]

    try:
        membres = [PortailUser.objects.get_from_ldap(uid) for uid in uids]
    except PortailUser.DoesNotExist:
        return mark_safe("<strong>{} :</strong> ???".format(titre))

    if affiche_ecole:
        if lien_annuaire:
            noms = ", ".join(["{} ({})".format(membre.get_link(), membre.ecole) for membre in membres])
        else:
            noms = ", ".join(["{} ({})".format(membre.get_full_name(), membre.ecole) for membre in membres])
    else:
        if lien_annuaire:
            noms = ", ".join([str(membre.get_link()) for membre in membres])
        else:
            noms = ", ".join([str(membre.get_full_name()) for membre in membres])

    if len(membres) == 1:
        return mark_safe("<strong>{} :</strong> {}".format(titre, noms))

    return mark_safe("<strong>{}s :</strong> {}".format(titre, noms))


def fiche(request, club_id):
    """Fiche d'un club"""
    try:
        club = Club.objects.par_ident(request.ecole, club_id)
    except Club.DoesNotExist:
        raise Http404

    club_ldap = club.get_ldap_club()
    aff_lien = request.user.is_authenticated
    ident = club_ldap.ident
    # ecole = Ecole.objects.get(ident=club_ldap.id_ecole)
    ecole = club.ecole
    proto = "https" if request.is_secure() else "http"

    # logo du club
    if club and club.logo:
        logo = club.logo
    else:
        logo = None

    # site web du club
    if club_ldap.hasWebsite:
        adresse = "{}://{}/clubs/{}/".format(proto, settings.URLS_SITES[ecole.id - 1], ident)
        lien_web = mark_safe(
            '<a class="text-decoration-none" href="{}"><i class="fas fa-globe-americas"></i> Site web</a>'.format(
                adresse
            )
        )
    else:
        lien_web = None

    # blog du club
    if club and club.blog_act:
        adresse = "{}://{}/clubs/{}/blog/".format(proto, settings.URLS_SITES[ecole.id - 1], ident)
        lien_blog = mark_safe(
            '<a class="text-decoration-none" href="{}"><i class="fas fa-feather"></i> Blog</a>'.format(adresse)
        )
    else:
        lien_blog = None

    # mail du club
    if club_ldap.mailAlias:
        # Mail encodé en rot13 et décodé en js lors du clic sur le lien pour éviter le spam
        obfuscated_mail = codecs.encode(f"mailto:{club_ldap.mailAlias[0]}", "rot_13")
        mail = format_html(
            '<a id="mail-club" class="text-decoration-none" href="#" onclick="showmail(\'{}\')">'
            '<i class="far fa-envelope"></i> Email</a>',
            obfuscated_mail,
        )
    else:
        mail = None

    # local du club
    if club_ldap.local:
        local = mark_safe('<i class="fas fa-map-marker-alt"></i> Local {}'.format(club_ldap.local))
    else:
        local = None

    # contacts
    _contacts = [lien_web, lien_blog, mail, local]
    nb_contacts = _contacts.count(None)
    # bureau (seulement ceux facultatifs)
    _bureau = [club_ldap.vicePresident, club_ldap.secretaire]
    nb_bureau = _bureau.count([])
    affiche_ecole = ecole == Ecole.inp

    club_infos = {
        "nb_contacts": "".join("x" for x in range(nb_contacts)),
        "nb_bureau": "".join("x" for x in range(nb_bureau)),
        "ident": ident,
        "logo": logo,
        "nom": club_ldap.displayName,
        "ecole": ecole,
        "type": TYPES[club_ldap.typeClub],
        "activite": TITRES[club_ldap.activite],
        "lien_web": lien_web,
        "lien_blog": lien_blog,
        "miniDescription": club.mini_description if club else "",
        "description": club.description if club else "",
        "president": responsable(club_ldap.president, "Président", aff_lien, affiche_ecole),
        "vicePresident": responsable(club_ldap.vicePresident, "Vice-président", aff_lien, affiche_ecole),
        "tresorier": responsable(club_ldap.tresorier, "Trésorier", aff_lien, affiche_ecole),
        "secretaire": responsable(club_ldap.secretaire, "Secrétaire", aff_lien, affiche_ecole),
        "mailAlias": mail,
        "local": local,
    }

    c = {
        "club": club_infos,
    }

    return render(request, "club/fiche.html", c)


def liste(request, id_page="tous"):
    """Liste des clubs"""
    filtre = None
    activites = {}

    # activité des clubs
    if id_page in ["tous", "ecoles", "inp"]:
        # toutes les activités
        for page in PAGES:
            contenu = PAGES[page]

            for activite in contenu:
                filtre |= L(activite=activite)

            if len(contenu) == 1:
                titre_clubs = TITRES[contenu[0]]
            else:
                titre_clubs = TITRES["autre"]
            activites[page] = {"titre": titre_clubs, "page": page, "liste_clubs": []}
    else:
        # une activité <id_page>
        try:
            contenu = PAGES[id_page]
        except KeyError:
            raise Http404

        # filtre activités
        for activite in contenu:
            filtre |= L(activite=activite)

        if len(contenu) == 1:
            titre_clubs = TITRES[contenu[0]]
        else:
            titre_clubs = TITRES["autre"]

        activites[id_page] = {"page": id_page, "titre": titre_clubs, "liste_clubs": []}

    # filtre écoles
    if id_page == "inp":
        filtre &= L(ecole=Ecole.inp)
        ecole_id = "inp"
    elif id_page == "ecoles":
        ecole_id = "toutes"
    else:
        ecole_id = ""
        filtre &= L(ecole=request.ecole)

    # récupération des clubs
    clubs = sorted(LdapClub.objects.filter(filtre), key=lambda c: c.displayName.lower())
    for club_ldap in clubs:
        ecole = Ecole.objects.get(ident=club_ldap.id_ecole)
        ident = club_ldap.ident
        try:
            club = Club.objects.get(ecole=ecole, ident=ident)
        except Club.DoesNotExist:
            club = None

        # logo du club
        if club and club.logo:
            logo = club.logo
        else:
            logo = None

        # url de la fiche du club
        if club:
            url = club.get_absolute_url()
        else:
            url = None

        for activite in PAGES:
            if club_ldap.activite in PAGES[activite]:
                page = activite

        activites[page]["liste_clubs"].append(
            {
                "ident": ident,
                "logo": logo,
                # 'est_inp': club_ldap.ecole.get() == Ecole.inp and request.ecole != Ecole.inp,
                "nom": club_ldap.displayName,
                "miniDescription": club.mini_description if club else "",
                "url": url,
            }
        )

    # recensement des activités à afficher
    index = []
    activites_ = sorted(activites.keys())  # tri des activités
    for activite in activites_:
        index.append(activites[activite])

    # TODO: index[activite].sort(key=lambda i: i['nom'].lower())

    c = {
        "activites": index,
        "ecole": ecole_id,
    }

    return render(request, "club/liste.html", c)


def article(request, club_id, adresse):
    club = get_object_or_404(Club, ecole=request.ecole, ident=club_id)
    article = get_object_or_404(ArticleBlog, club=club, adresse=adresse)

    # droit d'édition du blog du club
    ident_groupe = club_id + "-" + request.ecole.ident
    if request.user.is_authenticated:
        a_perm_clubs = request.user.check_permission(request, "gerer_clubs")
        est_dans_club = bool(LdapClub.objects.filter(cn=ident_groupe, memberUid=request.user.username))
        perm_blog = a_perm_clubs or est_dans_club
    else:
        perm_blog = None

    if not club.blog_act:
        raise Http404

    try:
        precedent = article.get_previous_by_date_post(club=club)
    except ArticleBlog.DoesNotExist:
        precedent = None

    try:
        suivant = article.get_next_by_date_post(club=club)
    except ArticleBlog.DoesNotExist:
        suivant = None

    c = {
        "perm_blog": perm_blog,
        "club": club,
        "article": article,
        "precedent": precedent,
        "suivant": suivant,
    }

    return render(request, "club/article.html", c)


def liste_articles(request, club_id, num_page=1):
    club = get_object_or_404(Club, ecole=request.ecole, ident=club_id)
    paginator = Paginator(ArticleBlog.objects.filter(club=club).order_by("-date_post"), 5)

    if not club.blog_act:
        raise Http404

    try:
        articles = paginator.page(num_page)
    except PageNotAnInteger:
        articles = paginator.page(1)
    except EmptyPage:
        articles = paginator.page(paginator.num_pages)

    c = {
        "club": club,
        "articles": articles,
        "url_kwargs": {"club_id": club_id},
    }

    return render(request, "club/liste_articles.html", c)


def rss(request, club_id):
    club = get_object_or_404(Club, ecole=request.ecole, ident=club_id)

    if not club.blog_act:
        raise Http404

    c = {
        "club": club,
        "articles": ArticleBlog.objects.filter(club=club).order_by("-date_post")[0:10],
    }

    return render(request, "club/rss.xml", c, content_type="application/rss+xml;encoding=UTF-8")
